class Author < ActiveRecord::Base
  has_one :artist
  belongs_to :user
  
  validates :name, :presence => true, :length => {:maximum => 30}
  validates :user_id, :numericality => true, :existence => true
  
  def supply_default
    self.name = 'no name' if self.name.blank?
  end
  
  def overwrite 
  end
  
  def own? au
    return false unless au.is_a?(Author)
    self.id == au.id
  end
  
  def visible? au
    if au == nil
      return false if MagicNumber['run_mode'] == 1
    elsif au.is_a?(Author)
      return true
    else
      return false
    end
    true
  end
  
  def artist?
    Artist.find_by_author(self) != nil
  end
  
  def step2 n
    self.name = n
    self.save
  end
  
  def self.default_page_size
    25
  end
  
  def self.max_page_size
    100
  end
  
  def self.page prm = nil
    page = prm.to_i
    page = 1 if page < 1
    page
  end
  
  def self.page_size prm = self.default_page_size
    page_size = prm.to_i
    page_size = self.max_page_size if page_size > self.max_page_size
    page_size = self.default_page_size if page_size < 1
    page_size
  end
  
  def self.list page = 1, page_size = self.default_page_size
    opt = {}
    opt.merge!(Author.list_opt)
    opt.merge!({:limit => page_size, :offset => (page -1) * page_size}) if page_size > 0
    opt.merge!({:order => 'created_at desc'})
    Author.find(:all, opt)
  end
  
  def self.list_opt
    {:include => {:artist => {}} }
  end
  
  def self.list_json_opt
    {:include => {:artist => {}} }
  end
  
  def self.show aid, au
    opt = {}
    opt.merge!(Author.show_opt)
    res = Author.find(aid, opt)
    raise ActiveRecord::Forbidden unless res.visible?(au)
    res
  end
  
  def self.show_opt
    {:include => {:artist => {}} }
  end
  
  def self.show_json_opt
    {:include => {:artist => {}} }
  end
  
  def self.edit aid, au
    opt = {}
    opt.merge!(Author.show_opt)
    res = Author.find aid, opt
    raise ActiveRecord::Forbidden unless res.own?(au)
    res
  end
  
  def self.default_comic_page_size
    25
  end
  
  def self.comic_max_page_size
    100
  end
  
  def self.comic_page_size prm = self.default_comic_page_size
    page_size = prm.to_i
    page_size = self.comic_max_page_size if page_size > self.comic_max_page_size
    page_size = self.default_comic_page_size if page_size < 1
    page_size
  end
  
  def self.default_story_page_size
    25
  end
  
  def self.story_max_page_size
    100
  end
  
  def self.story_page_size prm = self.default_story_page_size
    page_size = prm.to_i
    page_size = self.story_max_page_size if page_size > self.story_max_page_size
    page_size = self.default_story_page_size if page_size < 1
    page_size
  end
  
  def self.default_panel_page_size
    25
  end
  
  def self.panel_max_page_size
    100
  end
  
  def self.panel_page_size prm = self.default_panel_page_size
    page_size = prm.to_i
    page_size = self.panel_max_page_size if page_size > self.panel_max_page_size
    page_size = self.default_panel_page_size if page_size < 1
    page_size
  end
  
  def self.default_panel_picture_page_size
    25
  end
  
  def self.panel_picture_max_page_size
    100
  end
  
  def self.panel_picture_page_size prm = self.default_panel_picture_page_size
    page_size = prm.to_i
    page_size = self.panel_picture_max_page_size if page_size > self.panel_picture_max_page_size
    page_size = self.default_panel_picture_page_size if page_size < 1
    page_size
  end
  
  def self.default_ground_picture_page_size
    25
  end
  
  def self.ground_picture_max_page_size
    100
  end
  
  def self.ground_picture_page_size prm = self.default_ground_picture_page_size
    page_size = prm.to_i
    page_size = self.ground_picture_max_page_size if page_size > self.ground_picture_max_page_size
    page_size = self.default_ground_picture_page_size if page_size < 1
    page_size
  end
  
  def self.default_ground_color_page_size
    25
  end
  
  def self.ground_color_max_page_size
    100
  end
  
  def self.ground_color_page_size prm = self.default_ground_color_page_size
    page_size = prm.to_i
    page_size = self.ground_color_max_page_size if page_size > self.ground_color_max_page_size
    page_size = self.default_ground_color_page_size if page_size < 1
    page_size
  end
  
  def self.default_panel_color_page_size
    25
  end
  
  def self.panel_color_max_page_size
    100
  end
  
  def self.panel_color_page_size prm = self.default_panel_color_page_size
    page_size = prm.to_i
    page_size = self.panel_color_max_page_size if page_size > self.panel_color_max_page_size
    page_size = self.default_panel_color_page_size if page_size < 1
    page_size
  end
  
  def self.visible_count
    Author.count
  end
  
end
